;; -*- Scheme -*-
;;                Amended Functions.dsl

;
; Numbering Functions for section and footnote
;

(define (char-repeat sym num)
  (string-append (char-repeat sym (- num 1)) sym))

(define (num->alpha num)
  (substring " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
             num (+ num 1)))

(define (num->alphaCAP num)
  (substring " ABCDEFGHIJKLMNOPQRSTUVWXYZ"
             num (+ num 1)))

(define (num->kanji num) ;; Assume max 2 digit.
  (if (> num 10)
      (string-append (if (= (quotient num 10) 1)
                         ""
                         (num->kanji (quotient num 10)))
                     "\"
                     (num->kanji (remainder num 10)))
      (case num
        ((0) "Z") ((1) "") ((2) "") ((3) "O") ((4) "l") ((5) "")
        ((6) "Z") ((7) "") ((8) "") ((9) "") ((10) "\"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Roma numbering function
(define (num->roma num)
  (format-number num "i"))

(define (make-num n exp)
  (case exp
    (('abc) (num->alpha n))
    (('ABC) (num->alphaCAP n))
    (('kanji) (num->kanji n))
    (('roma) (num->roma n))
;    (('hira) (num->hira n))
;    (('kata) (num->hira n))
;    (('iroha-hira) (num->hira n))
;    (('iroha-kata) (num->hira n))
    (('asterisk)  (char-repeat "*" n))
    (('dag) (char-repeat "" n))  ;Dag, ddag, P, S and Vert follow TeX.
    (('ddag) (char-repeat "" n))
    (('P) (char-repeat "" n))
    (('S) (char-repeat "" n))
    (('Vert) (char-repeat "a" n))
    (('sharp)  (char-repeat "#" n))
    (else (number->string n))
))

(define (make-numbering nl desc)
(if (equal? #f desc)
  (make-numbering nl *title-number-desc*)
  (letrec ((num-member
            (lambda (e l)
              (cond ((not(list? l)) #f)
                    ((member e l) #t)
                    ((eq? 'nl (car (reverse l)))
                     (let* ((ll (reverse (cdr (reverse l)))))
                       (if (> (length ll) 0)
                           (> e (apply max ll))
                           #t)))
                    (else #f))))
           (match-level
            (lambda (level desc)
              (cond ((null? desc) #f)
                    ((not (list? (car desc))) desc)
                    ((not (list? (caar desc))) (car desc)) ; non-list
                    ((num-member level (caar desc)) (car desc))
                    (else (match-level level (cdr desc)))
                    )))
           (make-infix
            (lambda (nl num infix)
;             (cond ((eqv? 'last infix)                 to be revised
              (cond ((or (eqv? 'last infix) (eqv? "" infix))
                     (make-num (car (reverse nl)) num))
                    ((null? (cdr nl))
                     (make-num (car nl) num))
                    (else
                     (string-append
                      (make-num (car nl) num)
                      infix
                      (make-infix (cdr nl) num infix)))))))
    (cond ((number? nl) (make-numbering (list nl) desc))
          ((not (list? nl)) "")
          ((not (list? desc)) "")
          ((match-level (length nl) desc)
;           => (lambda (d)                             to be revised
            (let ((d (match-level (length nl) desc)))
                (string-append
                 (caddr d) ;; pre
                 (make-infix nl (cadr d)(cadddr d))
                 (cadddr (cdr d)) ;; post
                 )))
          (else "")))))

;; Paragraph

(define *paragraph-style* ;; Base set of paragraph
  (style
   font-size: *base-font-size*
   font-weight: *base-font-weight*
   font-posture: *base-font-posture*
   font-family-name: *base-font-family*
   line-spacing: (cadddr *page-spec*)
   quadding: 'start ;; Left alignment.
   ))

(define *fli-paragraph-style* ;; Indented paragraph
  (style
   use: *paragraph-style*
   first-line-start-indent: *first-line-start-indent*
   ))

(define (FIRST-ITEM-WITHOUT-LINEBREAK #!optional (snl (current-node)))
  (if (first-sibling? snl)
      (make sequence
        (literal (format-number (child-number snl) "&glyph;") ")")
        (process-children))
      (make paragraph
;        start-indent: (+ (inherited-start-indent) 10pt)
        (literal (format-number (child-number snl) "&glyph;") ")")
        (process-children))))

;; 4.29 First paragraph identification
(define (FIRST-PARAGRAPH-IDENTIFICATION)
  (make paragraph
    font-family-name: (if (first-sibling? (current-node))
                          "&FontName;"     ;Arbitrary fontname toward first paragraph
                          "&FontName;"     ;Arbitrary fontname toward after second paragraph
)))
; (define (FIRST-PARAGRAPH-IDENTIFICATION)
;   (if (first-sibling? (current-node))
;       (make paragraph
;         ;; first paragraph characteristic
;         ;; ...
;         )
;       (make paragraph
;         ;; other paragraphs characteristic
;         ;; ...
;         )))

;; 4.30 Paragraph separator
(define (PARAGRAPH-SEPARATOR)
  (make display-group
    (make leader (literal "&glyph;"))
    (make paragraph (process-children))
    (make leader (literal "&glyph;"))))

;; Headline/page number
;;     - See PAGE-HEADER and PAGE-FOOTER,  Here only style.

(define *header-footer-style*
  (style
   use: *paragraph-style*
   font-size: (* *base-font-size* 1.0)
   line-spacing: (* (cadddr *page-spec*) 1.0)
   ;;font-posture: 'italic

   ))

;; Footnote

(define *footnote-style*
  (style
   use: *paragraph-style*
   font-size: (* *base-font-size* 1.0)
   line-spacing: (* (cadddr *page-spec*) 1.0)
   ))

;; word-length adjustment

(define (word-length adjustment n)
  (make line-field
        field-width: (* *base-font-size* n) ;; n word-length adjustment
        (make paragraph
              quadding: 'justify
              last-line-quadding: 'justify
              (process-children))))

;; Caption

;;; node count function

(define (GET-HEADING-NUMS node-list)
;   (make-numbering (element-number-list node-list)))     to be revised
    (make-numbering (element-number-list node-list) #f))

;; Ruby

;;;;;;; improvement (glyph-annotation) ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (RUBY)
;  (make glyph-annotation                                                to be revised
;        annotation-glyph-placement: 'centered
;        annotation: (process-matching-children 'yomi)
;        (process-children)))
;
;(define (RUBY-ALTERNATIVE . f)
;  (let ((factor (if (null? f) *ruby-font-size-factor* (car f))))
;    (make paragraph
;          font-size: (* (inherited-font-size) factor)
;          (process-children))))

;;;;;; (glyph-annotation) ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (RUBY-ALTERNATIVE)
  (sosofo-append
    (literal "(")
    (process-children)
    (literal ")")))

;; superscript / subscript

;     period allows to omit the argument.
;     processor does not support the syntax.

;(define (SUBSCRIPT . f)                              to be revised
(define (SUBSCRIPT   f)
  (let ((factor (if (null? f) *subscript-font-size-factor* (car f))))
    (make math-sequence
          math-display-mode: 'inline
          (make subscript
                font-size: (* (inherited-font-size) factor)
                (process-children)))))

;(define (SUPERSCRIPT . f)                               to be revised
(define (SUPERSCRIPT   f)
  (let ((factor (if (null? f) *superscript-font-size-factor* (car f))))
    (make math-sequence
          math-display-mode: 'inline
          (make superscript
                font-size: (* (inherited-font-size) factor)
                (process-children)))))

;; Underline

(define (UNDERLINE)
  (make score
        type: 'after
        (process-children)))

;; THAI_Underline
(define (THAI_UNDERLINE)
  (make score
   type: 'after 
;; Note This specification is dependend to implementation.
;; When specified language is Thai and the value of type is eafter, 
;; underline must avoid if glyph is available on underline position
   (process-children)))

;; Inlinenote

;;;;;; improvement (multi-line-inline-node) ;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (INLINENOTE)
;  (make multi-line-inline-node ;; see ISO/IEC10179 12.6.24          to be revised
;        (process-children)))

;; Emphasized mark


;;;;;; improvement (emphasizing-mark) ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (EMPHASIZING-MARK)
  (make emphasizing-mark
    mark: (make mark
            (sosofo-label (literal "&dot;") 'over-mark)
            (sosofo-label (literal "&dot;") 'under-mark))
    ;;;; mark-style: Public Identifier ;;;;
    (process-children)))

;; Font / Typeface

(define (BOLD-SEQ)
  (make sequence
    font-weight: 'bold
    (process-children)))

(define (ITALIC-SEQ)
  (make sequence
    font-posture: 'italic
    (process-children)))

(define (BOLD-ITALIC-SEQ)
  (make sequence
        font-weight: 'bold
        font-posture: 'italic
        (process-children)))

(define (STRIKE-SEQ)
  (make score
    type: 'through
    (process-children)))

;; List

(define (MAKE-ENUM-EXP nul)
  (make-numbering nul *enum-number-desc*))


;;;;;;; to be revised ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (LIST-CONTAINER)
;  (make display-group
;        ;space-before:
;        ;space-after:
;        start-indent: (inherit-start-indent)
;))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (LIST-CONTAINER)
  (make display-group
        ;space-before:
        ;space-after:
        start-indent: (+ (inherited-start-indent) *indent-step*)
))

;;;;;;; to be revised ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (LIST-ELEMENT lhead)  ;; lhead -- list head string
;  (make paragraph
;        use: *paragraph-style*
;        ;space-before:
;        start-indent: (+ (inherit-start-indent) *indent-step*)
;        first-line-start-indent: (- *indent-step*)
;        (make line-field
;              field-width: *indent-step*
;              (literal lhead))
;        (process-children-trim)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (LIST-ELEMENT lhead)  ;; lhead -- list head string
  (make paragraph
        use: *paragraph-style*
        ;space-before:
        start-indent: (inherited-start-indent)
        first-line-start-indent: (- *indent-step*)
        (make line-field
              field-width: *indent-step*
              (literal lhead))
        (process-children-trim)))

;; 4.17.1 Ordered list
(define (NESTED-NUMBERING element-name #!optional (snl (current-node)))
  (let* ((numl (hierarchical-number-recursive element-name snl)) ; number list
         (len (length numl))
         (num (child-number snl)))
    (make paragraph
;      start-indent: (+ (inherited-start-indent) 10pt)
      (literal
       (let loop ((i len))
         (if (<= 0 i)
             (string-append (format-number num "&glyph;") (loop (- i 1)))
             ""))
       ") ")
      (process-children))))

;; 4.17.1 Ordered list with string bullet item
(define (STRING-BULLET-ELEMENT)
  (make paragraph
     use: *paragraph-style*
     ;;string-bullet-list: '(list("one" "two" "\"three\"" "four")).
    (process-children-trim)))

;; Title

;;;;;;; to be revised ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (TITLE-LARGE)
;  (make paragraph
;        use: *paragraph-style*
;        font-size: (list-ref *font-table* 2)
;        font-weight: 'bold
;        space-before:
;        (car
;         (case (gi (first (children (current-node))))
;           (("h2" "h3") (car (title-vertical-spacing "h1" 2)))
;           (else  (car (title-vertical-spacing "h1" 0)))))
;        space-after:
;        (cadr
;         (case (gi (first (children (current-node))))
;           (("h2" "h3") (cadr (title-vertical-spacing "h1" 2)))
;           (else  (cadr (title-vertical-spacing "h1" 0)))))
;        escapement-space-after: ;; used by character flow object class.
;        (title-char-spacing (count (children (current-node))) *paper-name*)
;        (literal (GET-HEADING-NUMS '("H1")))
;        (literal "  ")
;        (process-children)))

;;;;;;; to be revised ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (TITLE-MEDIUM)
;  (make paragraph
;        use: *paragraph-style*
;        font-size: (list-ref *font-table* 3)
;        font-weight: 'bold
;        space-before:
;        (let ((b1 (absolute-first-sibling? (current-node)))
;              (b2 (string=? (gi (first (children (current-node)))) "h3")))
;          (car
;           (cond
;            ((and b1 b2) (title-vertical-spacing "h2" 3))
;            ((and (not b1) b2) (title-vertical-spacing "h2" 2))
;            ((and b1 (not b2)) (title-vertical-spacing "h2" 1))
;            (else (title-vertical-spacing "h2" 0)))))
;        space-after:
;        (let ((b1 (absolute-first-sibling? (current-node)))
;              (b2 (string=? (gi (first (children (current-node)))) "h3")))
;          (cadr
;           (cond
;            ((and b1 b2) (title-vertical-spacing "h2" 3))
;            ((and (not b1) b2) (title-vertical-spacing "h2" 2))
;            ((and b1 (not b2)) (title-vertical-spacing "h2" 1))
;            (else (title-vertical-spacing "h2" 0)))))
;        escapement-space-after:
;        (title-char-spacing (count (children (current-node))) *paper-name*)
;  ;      (make embedded-text   ;; Title number                   to be revised
;              direction: 'right-to-left
;              escapement-space-after: 0
;              (literal (GET-HEADING-NUMS '(H1 H2)))
;              (literal "  ")
;               )
;        (process-children)))

;;;;;;; to be revised ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (TITLE-SMALL)
;  (make paragraph
;        use: *paragraph-style*
;        font-size: (list-ref *font-table* 4)
;        font-weight: 'bold
;        space-before:
;        (car
;         (if (absolute-first-sibling? (current-node))
;             (title-vertical-spacing "h2" 1)
;             (title-vertical-spacing "h2" 0)))
;        space-after:
;        (cadr
;         (if (absolute-first-sibling? (current-node))
;             (title-vertical-spacing "h2" 1)
;             (title-vertical-spacing "h2" 0)))
;        escapement-space-after:
;        (title-char-spacing (count (children (current-node))) *paper-name*)
; ;       (make embedded-text   ;; Title number                 to be revised
;              direction: 'right-to-left
;              escapement-space-after: 0
;              (literal (GET-HEADING-NUMS '(H1 H2 H3)))
;              (literal "  ")
;               )
;        (process-children)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (TITLE-LARGE)
  (make paragraph
        use: *paragraph-style*
        font-size: (list-ref (list-ref *font-table* 1) 1)
        font-weight: 'bold
        space-before:
        (car
         (case (gi (first (children (current-node))))
           (("h2" "h3") (title-vertical-spacing "h1" 2))
           (else (title-vertical-spacing "h1" 0))))
        space-after:
        (cadr
         (case (gi (first (children (current-node))))
           (("h2" "h3") (title-vertical-spacing "h1" 2))
           (else (title-vertical-spacing "h1" 0))))
        escapement-space-after:
           (title-char-spacing (count (children (current-node))) *paper-name*)
        (literal (GET-HEADING-NUMS '("H1")))
    (literal "  ")
        (process-children)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (TITLE-MEDIUM)
  (make paragraph
        use: *paragraph-style*
        font-size: (list-ref (list-ref *font-table* 2) 1)
        font-weight: 'bold
        space-before:
        (let ((b1 (absolute-first-sibling? (current-node)))
              (b2 (string=? (gi (first (children (current-node)))) "h3")))
          (car
           (cond
            ((and b1 b2) (title-vertical-spacing "h2" 3))
            ((and (not b1) b2) (title-vertical-spacing "h2" 2))
            ((and b1 (not b2)) (title-vertical-spacing "h2" 1))
            (else (title-vertical-spacing "h2" 0)))))
        space-after:
        (let ((b1 (absolute-first-sibling? (current-node)))
              (b2 (string=? (gi (first (children (current-node)))) "h3")))
          (cadr
           (cond
            ((and b1 b2) (title-vertical-spacing "h2" 3))
            ((and (not b1) b2) (title-vertical-spacing "h2" 2))
            ((and b1 (not b2)) (title-vertical-spacing "h2" 1))
            (else (title-vertical-spacing "h2" 0)))))
        escapement-space-after:
        (title-char-spacing (count (children (current-node))) *paper-name*)
        (literal (GET-HEADING-NUMS '("H1" "H2")))
        (literal "  ")
        (process-children)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (TITLE-SMALL)
  (make paragraph
        use: *paragraph-style*
        font-size: (list-ref (list-ref *font-table* 3) 1)
        font-weight: 'bold
        space-before:
        (car
         (if (absolute-first-sibling? (current-node))
             (title-vertical-spacing "h2" 1)
             (title-vertical-spacing "h2" 0)))
        space-after:
        (cadr
         (if (absolute-first-sibling? (current-node))
             (title-vertical-spacing "h2" 1)
             (title-vertical-spacing "h2" 0)))
        escapement-space-after:
        (title-char-spacing (count (children (current-node))) *paper-name*)
        (literal (GET-HEADING-NUMS '("H1" "H2" "H3")))
        (literal "  ")
        (process-children)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Some additional definitions
(define (caar a)
    (car (car a)))

(define (cadr a)
    (car (cdr a)))

(define (caddr a)
    (car (cdr (cdr a))))

(define (cadddr a)
    (car (cdr (cdr (cdr a)))))

(define (first a)
    (node-list-first a))

(define (eq? a b)
        (if (and (not (list? a)) (not (list? b)))
        (equal? a b)
        #f))

(define (eqv? a b)
    (eq? a b))

(define (count nl)
     (node-list-count nl))

(define (node-list-count nl)
  (node-list-length (node-list-remove-duplicates nl)))

(define (node-list-remove-duplicates nl)
  (node-list-reduce nl
                    (lambda (result snl)
                      (if (node-list-contains? result snl)
                          result
                          (node-list result snl)))
                    (empty-node-list)))

(define (node-list-contains? nl snl)
  (node-list-reduce nl
                    (lambda (result i)
                      (or result
                          (node-list=? snl i)))
                    #f))

(define (string=? a b)
    (if (and (string? a) (string? b))
        (equal? a b)
        #f))

(define (ENCLOSURE)
  (make box
    (
;;     box-size-before: specification of the length between  before line and placement-path
;;     box-size-after: specification of the length between  after line and placement-path
;;     box-size-start: specification of the length between  end line and first glyph
;;     box-size-end: specification of the length between  start line and last glyph
       box-open-end?: #t
      (process-children))))

;; NOTE Dropcap specification using the flow object class of table. 

(define (dropcap scale num str width)
  ;; width is number of characters in one line
  (let* ((len (string-length str))
         (fsize (* *font-size* scale))
         (b (* (- width (* scale num)) scale))
         (a (if (< b len) (+ b num) len))
        )
    (make table
      (make table-column width: (* fsize num))
      (make table-column)
      (make table-row
        (make table-cell
          (make paragraph
            line-spacing: (if (< *line-spacing* fsize) fsize *line-spacing*)
            font-size: fsize
            (literal (substring str 0 num))))
        (make table-cell
          (make paragraph
            (literal (substring str num a)))))
      (make table-row
        (make table-cell
          n-columns-spanned: 2
          (make paragraph
            (literal (substring str a len)))))
      )))

;; NOTE Dropcap specification using the characteristic of dropcap introduced in Amendment 2 to ISO/IEC 10179

(define (DROPCAP)
  (make paragraph
    ;; drop-character-number: specification of the number of characters can be displayed as drop cap characters
    ;; dropcap-height: specification of height of a dropcap region expressed by pt
    ;; dropcap-width: specification of width of a dropcap region expressed by pt
   ))
